#############
# $Id: reorder_fastq.pm 12454 2011-01-26 10:59:47Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2009-09-16
# Last Changed On: $Date: 2011-01-26 10:59:47 +0000 (Wed, 26 Jan 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-24.1/lib/npg_pipeline/reorder_fastq.pm $

package npg_pipeline::reorder_fastq;
use Moose;
use MooseX::InsideOut;
use Carp qw{cluck carp confess croak};
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12454 $ =~ /(\d+)/mxs; $r; };
use Perl6::Slurp;

extends q{npg_pipeline::base};

# we are happy with these here, since this was for a specific potential problem which has gone away
Readonly::Scalar our $MOD_FOUR    => 4;
Readonly::Scalar our $SLEEPY_TIME => 5;

has q{path}           => (isa => q{Str},      is => q{ro}, required   => 1, documentation => q{path/to/find/fastq/files});
has q{new_path}       => (isa => q{Str},      is => q{ro}, predicate  => q{has_new_path}, documentation => q{optional - path/to/put/corrected/fastq/files});
has q{fastq1}         => (isa => q{Str},      is => q{ro}, required   => 1, documentation => q{fastq1_file});
has q{fastq2}         => (isa => q{Str},      is => q{ro}, required   => 1, documentation => q{fastq2_file});
has q{fastqt}         => (isa => q{Str},      is => q{ro}, predicate  => q{has_fastqt}, documentation => q{optional - fastqt_file});

has q{tile_order}     => (isa => q{ArrayRef}, is => q{ro}, lazy_build => 1, init_arg   => undef);
has q{fastq_blocks}   => (isa => q{HashRef},  is => q{ro}, lazy_build => 1, init_arg   => undef);
has q{timestamp}      => (isa => q{Int},      is => q{ro}, lazy_build => 1, init_arg   => undef);
has q{_fastq1_count}  => (isa => q{Int},      is => q{rw}, init_arg   => undef);
has q{_fastq_count}   => (isa => q{Int},      is => q{rw}, init_arg   => undef);


sub run {
  my ($self) = @_;

  my $fastq1 = $self->path() . q{/} . $self->fastq1();
  my $fastq2 = $self->path() . q{/} . $self->fastq2();

  $self->_get_tile_order($fastq1);

  $self->_get_fastq_blocks($fastq2);
  if ($self->_fastq1_count() != $self->_fastq_count()) {
    croak q{fastq1 line count } . $self->_fastq1_count() . q{doesn't equal fastq2 line count } . $self->_fastq_count();
  }

  if (!$self->has_new_path()) {
    $self->_move_old_fastq($fastq2);
  }
  $self->_write_fastq($self->fastq2());

  if ($self->has_fastqt()) {

    my $fastqt = $self->path() . q{/} . $self->fastqt();

    $self->_get_fastq_blocks($fastqt);

    if ($self->_fastq1_count() != $self->_fastq_count()) {
      croak q{fastq1 line count } . $self->_fastq1_count() . q{doesn't equal fastqt line count } . $self->_fastq_count();
    }

    if (!$self->has_new_path()) {
      $self->_move_old_fastq($fastqt);
    }

    $self->_write_fastq($self->fastqt());
  }

  return 1;
}

sub _write_fastq {
  my ($self, $fastq) = @_;
  my $path = $self->has_new_path() ? $self->new_path()
           :                         $self->path()
           ;
  $fastq = $path . q{/} . $fastq;
  open my $fh, q{>}, $fastq or croak qq{Could not open $fastq to write};
  foreach my $tile (@{$self->tile_order}) {
    my $filename = $path . q{/} . $self->timestamp() . q{_} . $tile;
    my $fq_block = slurp $filename;
    print {$fh} $fq_block || croak qq{Unable to write block for tile $tile};
  }
  close $fh || carp q{Could not close FileHandle};
  $self->clear_fastq_blocks();
  carp qq{Written $fastq\n};
  sleep $SLEEPY_TIME;
  return 1;
}

sub _store_out_block {
  my ($self, $block, $tile) = @_;
  my $path = $self->path();
  if ($self->has_new_path()) {
    $path = $self->new_path();
  }
  my $filename = $path . q{/} . $self->timestamp() . q{_} . $tile;
  open my $fh, q{>}, $filename or croak qq{Could not open $filename for writing.};
  print {$fh} $block || croak qq{could not write out $block to file};
  close $fh || carp qq{could not close $fh};
  return 1;
}

sub _move_old_fastq {
  my ($self, $fastq) = @_;
  my $old_fq = $fastq . q{old};
  my $return = `mv $fastq $old_fq`;
  carp qq{moved $fastq to $old_fq\n};
  return $return;
}

sub _get_tile_order {
  my ($self, $fastq) = @_;
  my $tile_order = $self->tile_order();
  my $count = 0;
  my $last_tile_inserted;
  open my $fh, q{<}, $fastq or croak qq{Couldn't open $fastq for reading'};
  while ( readline $fh ) {
    my $line = $_;
    if (($count % $MOD_FOUR) != 0) {
      $count++;
      next;
    }
    my $tile = $self->_identify_tile($line);
    if (!$last_tile_inserted || $tile != $last_tile_inserted) {
      push @{$tile_order}, $tile;
      $last_tile_inserted = $tile;
    }
    $count++;
  }
  close $fh || croak qq{Unable to close filehandle for $fastq};
  $self->_fastq1_count($count);
  my $tile_order_string = join q{, }, @{$tile_order};
  carp qq{Order of $count tiles in $fastq is : } . $tile_order_string . qq{\n};
  sleep $SLEEPY_TIME;
  return 1;
}

sub _get_fastq_blocks {
  my ($self, $fastq) = @_;
  my $fastq2_blocks = $self->fastq_blocks();
  my $count = 0;
  my $current_tile;
  open my $fh, q{<}, $fastq or croak qq{Couldn't open $fastq for reading'};
  while ( readline $fh ) {
    my $line = $_;
    if (($count % $MOD_FOUR) != 0) {
      $fastq2_blocks->{$current_tile} .= $line;
      $count++;
      next;
    }
    my $tile = $self->_identify_tile($line);
    if (!$current_tile || $tile != $current_tile) {
      $fastq2_blocks->{$tile} = $line;
      if ($current_tile) {
        $self->_store_out_block($fastq2_blocks->{$current_tile}, $current_tile);
        delete $fastq2_blocks->{$current_tile};
      }
      $current_tile = $tile;
    } else {
      $fastq2_blocks->{$current_tile} .= $line;
    }
    $count++;
  }
  close $fh || carp qq{Could not close filehandle for $fastq};
  $self->_store_out_block($fastq2_blocks->{$current_tile}, $current_tile);
  $self->_fastq_count($count);
  carp qq{Processed fastq_blocks for $fastq\n};
  sleep $SLEEPY_TIME;
  return 1;
}


sub _identify_tile {
  my ($self, $line) = @_;
  my @temp = split /:/xms, $line;
  return $temp[2];
}

sub _build_fastq_blocks {
  my ($self) = @_;
  return {};
}

sub _build_tile_order {
  my ($self) = @_;
  return [];
}

sub _build_timestamp {
  my ($self) = @_;
  return time;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::reorder_fastq

=head1 VERSION

$LastChangedRevision: 12454 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 run - method to run and therfore do everything

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item Perl6::Slurp

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
