#############
# $Id: AllRunInfo.pm 12454 2011-01-26 10:59:47Z ajb $
# Created By: ajb
# Last Maintained By: $Author: ajb $
# Created On: 2010-06-15
# Last Changed On: $Date: 2011-01-26 10:59:47 +0000 (Wed, 26 Jan 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-24.1/lib/npg_pipeline/spider/AllRunInfo.pm $

package npg_pipeline::spider::AllRunInfo;
use Moose;
use MooseX::InsideOut;
use Carp qw{ carp confess croak cluck};
use English qw{-no_match_vars};
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 12454 $ =~ /(\d+)/mxs; $r; };

use st::api::util;
use npg::api::util;
use npg::api::run;
use st::api::batch;
use st::api::asset;
use npg_common::sequence::reference;

extends qw{npg_pipeline::base};
with q{npg_common::roles::run::path_info};
with q{npg_common::roles::run::long_info};
with q{npg_common::roles::run::fs_resource};
with q{npg_pipeline::roles::business::Spidering};
with q{npg_pipeline::roles::business::flag_options};

# these are required for use, so will not be put in a conf file
Readonly::Scalar our $NPG_API_RUN_PACKAGE_NAME  => q{npg::api::run};
Readonly::Scalar our $ST_API_BATCH_PACKAGE_NAME => q{st::api::batch};

=head1 NAME

npg_pipeline::spider::AllRunInfo

=head1 VERSION

$LastChangedRevision: 12454 $

=head1 SYNOPSIS

  use npg_pipeline::spider::AllRunInfo;

  my $oSpiderInfo = npg_pipeline::spider::AllRunInfo->new( $options_hash );

=head1 DESCRIPTION

This module is meant to operate and go spider for all information that the pipeline is going to want.
The primary reason so that it might be cached, and then retrieved from the cache locationm rather than
other sources (mostly web services), therefore reducing the dependencies for bsub jobs to ideally
just the filesystem, and maybe another cache source (Memcached or DB).

=head1 SUBROUTINES/METHODS

=head2 BUILD

This method just ensures that you have provided something that can retrieve an id_run from on construction

=cut

sub BUILD {
  my ( $self ) = @_;
  if ( ! ( $self->has_id_run()
            ||
           $self->has_run_folder()
            ||
           $self->has_name() ) ) {
    croak q{id_run not provided};
  }

  eval {
    require $self->convert_package_name_to_filename( $NPG_API_RUN_PACKAGE_NAME );
    require $self->convert_package_name_to_filename( $ST_API_BATCH_PACKAGE_NAME );
  } or do {
    croak q{Unable to require basic package name: } . $EVAL_ERROR;
  };

  return 1;
}

=head2 spider_all

goes out and spiders all the information it thinks it needs for a run

=cut

sub spider_all {
  my ( $self ) = @_;

  my $id_run = $self->id_run();

  eval {
    my $run = $self->spider_npg_run( $id_run );
    my $batch = $self->spider_sequencescape_batch( $run->batch_id() );
  } or do {
    confess $EVAL_ERROR;
  };

  return 1;
}

=head2 spider_npg_run

goes out and spiders, for npg run info (note, this does include some sequencescape as some is quite common)
returns the top level run object

=cut

sub spider_npg_run {
  my ( $self, $id_run ) = @_;

  my $run;
  eval {
    $run = $self->retrieve_all_npg( $id_run, $NPG_API_RUN_PACKAGE_NAME );
  } or do {
    confess $EVAL_ERROR;
  };
  return $run;
}

=head2 spider_sequencescape_batch

goes out and spiders, only sequencescape info is located.
Returns the top level batch object

=cut

sub spider_sequencescape_batch {
  my ( $self, $batch_id ) = @_;

  my $batch;
  eval {
    $batch = $self->retrieve_all_sequencescape( $batch_id, $ST_API_BATCH_PACKAGE_NAME );
  } or do {
    confess $EVAL_ERROR;
  };
  return $batch;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item Carp

=item English -no_match_vars

=item Readonly

=item npg_common::roles::run::path_info

=item npg_common::roles::run::long_info

=item npg_common::roles::run::fs_resource

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
