# $Id: 20-archive-SchemaInformation.t 12454 2011-01-26 10:59:47Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 85;
use Test::Exception::LessClever;
use Test::MockObject;
use File::Slurp;

my ( $mock, $version );

BEGIN {
  $mock = Test::MockObject->new();
  $mock->mock( q{aligner_version}, sub { return q{1.0}; } );
  $mock->fake_module( q{npg_qc::autoqc::checks::check} );
  $mock->fake_new( q{npg_qc::autoqc::checks::check} );

  use_ok( q{npg_pipeline::archive::file::SchemaInformation} );
}


$version = q{1.0};

use lib qw{t};
use t::util;
my $util = t::util->new();

my $paths = $util->set_rta_staging_analysis_area();
my $analysis_runfolder_path = $paths->{runfolder_path};
my $runfolder = $util->test_run_folder();
my $basecalls = $analysis_runfolder_path . q{/Data/Intensities/BaseCalls};
qx{mkdir -p $basecalls/log};
qx{mkdir -p $basecalls/PB_cal/log};
qx{mkdir -p $basecalls/PB_cal/archive};

$ENV{TEST_USERAGENT} = $analysis_runfolder_path . q{/useragent};
write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );


$ENV{TEST_BIN} = q{t/bin/};

{
  my $schema_info;
  lives_ok {
    $schema_info = npg_pipeline::archive::file::SchemaInformation->new( {
      run_folder => $runfolder,
      runfolder_path => $analysis_runfolder_path,
      lane => 1,
      conf_path => q{data/config_files},
      domain => q{test},
      no_recalibration => 1,
      no_srf_generation => 1,
      no_bam_generation => 1,
    } );
  } q{create object ok};
  isa_ok( $schema_info, q{npg_pipeline::archive::file::SchemaInformation}, q{$schema_info} );

  is_deeply( $schema_info->_make_gerald_tiles(), {}, q{no_recalibration -> no gerald data} );
  is( $schema_info->_first_program_details(), undef, q{no_recalibration -> no program details returned for gerald} );
  is_deeply( $schema_info->_create_srf(), {}, q{no_srf_generation -> no create_srf data} );
  is( $schema_info->_first_program_details(), undef, q{no_srf_generation -> no program details returned for create_srf} );
  is_deeply( $schema_info->_bam_generation(), {}, q{no_bam_generation -> no bam_generation data} );
  is( $schema_info->_first_program_details(), undef, q{no_bam_generation -> no program details returned for bam_generation} );
  is_deeply( $schema_info->_bam_markduplicate(), {}, q{no_bam_generation -> no bam_markduplicate data} );
  is( $schema_info->_first_program_details(), undef, q{no_bam_generation -> no program details returned for bam_markduplicate} );
  is_deeply( $schema_info->_demultiplex(), {}, q{run is not indexed -> no demultiplex data} );
  is( $schema_info->_first_program_details(), undef, q{run is not indexed -> no program details returned for demultiplex} );
  is_deeply( $schema_info->_split_fastq_by_tag(), {}, q{run is not indexed -> no split_fastq_by_tag data} );
  is( $schema_info->_first_program_details(), undef, q{run is not indexed -> no program details returned for split_fastq_by_tag} );
  is_deeply( $schema_info->_harold_alignment_files(), {}, q{run is not indexed -> no harold_alignment_files data} );
  is( $schema_info->_first_program_details(), undef, q{run is not indexed -> no program details returned for harold_alignment_files} );
  is_deeply( $schema_info->_harold_calibration_tables(), {}, q{run is not indexed -> no harold_calibration_tables data} );
  is( $schema_info->_first_program_details(), undef, q{run is not indexed -> no program details returned for harold_calibration_tables} );
  is_deeply( $schema_info->_harold_recalibration(), {}, q{run is not indexed -> no harold_recalibration data} );
  is( $schema_info->_first_program_details(), undef, q{run is not indexed -> no program details returned for harold_recalibration} );
#  is_deeply( $schema_info->_split_nonconsented_sequence(), {}, q{lane no unconsented data -> no split_nonconsented_sequence data} );
#  is( $schema_info->_first_program_details(), undef, q{lane no unconsented data -> no program details returned for split_nonconsented_sequence} );

  
  my $log_file_hash;
  lives_ok {
    $log_file_hash = $schema_info->_parse_run_log( q{t/data/logs/log_test_20090811-110301.log} );
  } q{_parse_run_log runs ok};
  isa_ok( $log_file_hash, q{HASH}, q{_parse_run_log} );
}

{
  my $schema_info = npg_pipeline::archive::file::SchemaInformation->new( {
      run_folder => $runfolder,
      runfolder_path => $analysis_runfolder_path,
      lane => 1,
      conf_path => q{data/config_files},
      domain => q{test},
      no_control_lane => 1,
      is_indexed => 1,
  } );

  qx{touch $analysis_runfolder_path/lane_1.tag};

  is_deeply( $schema_info->_make_gerald_tiles(), {}, q{no_control_lane -> no gerald data} );
  is( $schema_info->_first_program_details(), undef, q{no_control_lane -> no program details returned for gerald} );

  is_deeply( $schema_info->_demultiplex(), {}, q{lane is not indexed -> no demultiplex data} );
  is( $schema_info->_first_program_details(), undef, q{lane is not indexed -> no program details returned for demultiplexing} );

  is_deeply( $schema_info->_split_fastq_by_tag(), {}, q{lane is not indexed -> no split_fastq_by_tag data} );
  is( $schema_info->_first_program_details(), undef, q{lane is not indexed -> no program details returned for split_fastq_by_tag} );
}

{
  my $schema_info;
  lives_ok {
    $schema_info = npg_pipeline::archive::file::SchemaInformation->new( {
      run_folder => $runfolder,
      runfolder_path => $analysis_runfolder_path,
      lane => 1,
      conf_path => q{data/config_files},
      domain => q{test},
      is_indexed => 1,
      post_qseq_log_file => q{t/data/logs/log_test_20090811-110301.log},
      autoqc_checks_check => $mock,
      recalibrated_path => $paths->{recalibrated_path},
      aligner_cmd => q{t/bin/software/solexa/bin/bwa},
      samtools_cmd => q{t/bin/software/solexa/bin/samtools},
    } );
  } q{create object ok};
  isa_ok( $schema_info, q{npg_pipeline::archive::file::SchemaInformation}, q{$schema_info} );

  my $archive_path = $schema_info->archive_path();

  $schema_info->make_log_dir( $archive_path ); # make sure that an archive directory has been created

#  qx{cp t/data/logs/fastq_creation_1234_20101115-114710.1.123456.out $archive_path/};

#  $file = read_file( qq{$archive_path/fastq_creation_1234_20101115-114710.1.123456.out} );

#  diag $file;
#  die;
  my $info = [
    {
      PN => q{test_one},
      PP => undef,
      VN => 0,
      CL => undef,
    },
    {
      PN => q{test_two},
      PP => q{test_one},
      VN => q{1.4},
      CL => q{test_two --id_run 1234 --lane 1},
    },
    {
      PN => q{test_four},
      PP => q{test_three},
      VN => 1,
    },
    {
      PN => q{test_three},
      PP => q{test_two},
      VN => q{0,3},
    },
    
  ];

  lives_ok {
    $schema_info->_write_to_file( $info );
  } q{information written to file ok};

  my $info_retrieved;
  lives_ok {
    $info_retrieved = $schema_info->read_in_from_file();
  } q{file read ok};
  is_deeply( $info_retrieved, $info, q{retrieved data structure is same as original data structure} );

  my $hash = $schema_info->_run_bustard();
  is_deeply( $hash, {
    ID => q{basecalling},
    PN => q{Bustard},
    VN => q{1.8.1a2},
    DS => q{the basecalling package(OLB - Bustard) package used},
  }, q{Bustard data retrieved correctly} );

  is( $schema_info->_first_program_details(), $hash, q{_bustard stored in the _information array OK} );

  $hash = $schema_info->_make_gerald_tiles();
  is_deeply( $hash, {
    ID => q{recalibration},
    PN => q{Gerald},
    VN => q{@(#) Id: GERALD.pl,v 1.15 2009/11/04 10:52:21 craczy Exp(/software/solexa/src/CASAVA_v1.6.0a10-build/bin/../bin)},
    DS => q{the recalibration CASAVA(Gerald) package used},
  }, q{Gerald data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_make_gerald_tiles stored in the _information array OK} );

  $hash = $schema_info->_rta();
  is_deeply( $hash, {
    ID => q{SCS},
    PN => q{RTA},
    VN => q{1.8.70.0},
    DS => q{Controlling software on instrument},
  }, q{RTA data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_rta stored in the _information array OK} );

  my $file = read_file( q{t/data/logs/fastq_creation_1234_20101115-114710.1.123456.out} );
  $file =~ s/ARCHIVE_PATH/$archive_path/gxms;
  $file =~ s/RECAL_PATH/$paths->{recalibrated_path}/gxms;
  write_file( $archive_path . q{/log/fastq_creation_1234_20101115-114710.1.123456.out}, $file );

  $hash = $schema_info->_create_fastq();
  is_deeply( $hash, {
    ID => q{fastq_creator},
    PN => q{qseq2fastq.pl},
    VN => q{20090716.01},
    PP => undef,
    DS => q{the program which converts the qseq files to fastq files},
    CL => q{/software/solexa/bin/qseq2fastq.pl --md5 <md5_name> --fastqcheck <fastqcheck_name> --filter --unique <qseq_files> > <fastq_name>},
  }, q{create_fastq data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_create_fastq stored in the _information array OK} );

  $hash = $schema_info->_create_srf();
  is_deeply( $hash, {
    ID => q{srf_creator},
    PN => q{illumina2srf},
    VN => q{2.1.1},
    PP => undef,
  }, q{create_srf data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_create_srf stored in the _information array OK} );

  $hash = $schema_info->_bam_generation();
  is_deeply( $hash, {
    ID => q{fastq_to_bam_wrapper},
    PN => q{fastq2bam.pl},
    VN => q{9166},
    DS => q{the program used to convert fastq to bam},
  }, q{bam_generation data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_bam_generation stored in the _information array OK} );
  is( $schema_info->_count_program_details(), 6, q{bam_generation has created correct number of lots of details} );
  $schema_info->_clear_program_details();

  $hash = $schema_info->_bam_markduplicate();
  is_deeply( $hash, {
    ID => q{bam_markduplicate_wrapper},
    PN => q{bam_mark_duplicate.pl},
    VN => q{9783},
    DS => q{the script used to determine duplicate reads within the bam file},
  }, q{bam_markduplicate data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_bam_markduplicate stored in the _information array OK} );

  qx{cp t/data/tag_files/lane_7.tag $analysis_runfolder_path/lane_1.tag};

  $hash = $schema_info->_demultiplex();
  is_deeply( $hash, {
    ID => q{demultiplex},
    PN => q{index_decoder},
    VN => q{2.1.3 (r93)},
    DS => q{the script which processes tag sequences to determine the sample that the sequence belongs to},
  }, q{demultiplex data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_demultiplex stored in the _information array OK} );

  $hash = $schema_info->_split_fastq_by_tag();
  is_deeply( $hash, {
    ID => q{split_fastq_by_tag},
    PN => q{split_fastq_by_tag},
    VN => undef,
  }, q{split_fastq_by_tag data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_split_fastq_by_tag stored in the _information array OK} );

  $schema_info->_lane_splitting_cache()->{1} = 'true';
  my $archive_log_dir = $schema_info->make_log_dir( qq{$paths->{recalibrated_path}/archive} );
  qx{cp t/data/logs/split_nonconsented_1234_20101119-022433.1.829471.out $archive_log_dir/};
  $hash = $schema_info->_split_nonconsented_sequence();
  my $fastq_split_version = npg_common::Fastq_Split->VERSION;
  is_deeply( $hash, {
    ID => q{split_nonconsented_sequence},
    PN => q{fastq_split},
    VN => $fastq_split_version,
    CL => q{/software/solexa/bin/fastq_srf_split.pl -fastq1 archive/1234_1_1.fastq -fastq2 archive/1234_1_2.fastq -fork_alignment -id_run 1234 -position 1 -output_dir archive && mv && mv again},
    DS => q{splits out non_consented DNA sequences from the sequenced data},
  }, q{split_nonconsented_sequence data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_split_nonconsented_sequence stored in the _information array OK} );

  $schema_info->_lane_phix_splitting_cache()->{1} = 'true';
  qx{cp t/data/logs/split_nonconsented_1234_20101119-022433.1.829471.out $archive_log_dir/split_phix_1234_20101119-022433.1.829472.out};
  $hash = $schema_info->_split_phix();
  is_deeply( $hash, {
    ID => q{split_spiked_phix},
    PN => q{fastq_split},
    VN => $fastq_split_version,
    CL => q{/software/solexa/bin/fastq_srf_split.pl -fastq1 archive/1234_1_1.fastq -fastq2 archive/1234_1_2.fastq -fork_alignment -id_run 1234 -position 1 -output_dir archive},
    DS => q{splits out spiked phix out from the sequenced data},
  }, q{split_phix data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_split_phix stored in the _information array OK} );

  $hash = $schema_info->_create_md5();
  is_deeply( $hash, {
    ID => q{md5_creator},
    PN => q{md5sum},
    VN => q{5.97},
    PP => undef,
  }, q{create_md5 data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_create_md5 stored in the _information array OK} );

  $hash = $schema_info->_create_fastqcheck();
  is_deeply( $hash, {
    ID => q{fastqcheck_creator},
    PN => q{fqcheck_creator},
#    VN => undef, # currently not listed
    PP => undef,
  }, q{create_fastqcheck data retrieved correctly} );
  is( $schema_info->_first_program_details(), $hash, q{_create_fastqcheck stored in the _information array OK} );

  lives_ok {
    $schema_info->_build_job_details();
  } q{_build_job_details runs ok};

  my @hrefs = (
     {
       'ID' => 'SCS', 'VN' => '1.8.70.0', 'PN' => 'RTA', 'DS' => q'Controlling software on instrument',
     },
     {
       'ID' => 'post_qseq', 'PN' => 'npg_pipeline_post_qseq', 'VN' => '10107',
     },
     {
       'ID' => 'create_archive_directory', 'PN' => 'create_archive_directory', 'PP' => undef, 'VN' => undef
     },
     {
       'ID' => 'srf_creator', 'PN' => 'illumina2srf', 'PP' => 'create_archive_directory', 'VN' => '2.1.1',
     },
     {
       'ID' => 'fastq_creator', 'PN' => 'qseq2fastq.pl', 'PP' => 'create_srf', 'VN' => '20090716.01', 'DS' => 'the program which converts the qseq files to fastq files', 'CL' => '/software/solexa/bin/qseq2fastq.pl --md5 <md5_name> --fastqcheck <fastqcheck_name> --filter --unique <qseq_files> > <fastq_name>',
     },
     {
       'ID' => 'fastqcheck_creator', 'PN' => 'fqcheck_creator', 'PP' => 'create_fastq',
     },
     {
       'ID' => 'create_gcfreq', 'PN' => 'create_gcfreq', 'PP' => 'create_fastqcheck', 'VN' => undef
     },
     {
       'ID' => 'md5_creator', 'PN' => 'md5sum', 'PP' => 'create_gcfreq', 'VN' => '5.97'
     },
  );

  while ( my $href = $schema_info->_first_program_details() ) {
    is_deeply( $href, shift @hrefs, q{ok} );
  }
}
{
  my $schema_info;
  lives_ok {
    $schema_info = npg_pipeline::archive::file::SchemaInformation->new( {
      run_folder => $runfolder,
      runfolder_path => $analysis_runfolder_path,
      lane => 1,
      conf_path => q{data/config_files},
      domain => q{test},
      is_indexed => 1,
      analyse_rta_log_file => q{t/data/logs/analyse_RTA_json_part.log},
      post_qseq_log_file => q{t/data/logs/post_qseq_json_part.log},
      recalibrated_path => $paths->{recalibrated_path},
      aligner_cmd => q{t/bin/software/solexa/bin/bwa},
      samtools_cmd => q{t/bin/software/solexa/bin/samtools},
    } );
  } q{create object ok};
  $schema_info->_lane_splitting_cache()->{1} = 'true';
  $schema_info->_lane_phix_splitting_cache()->{1} = 'true';
  lives_ok {
    $schema_info->_build_job_details();
  } q{_build_job_details runs ok};

  my $count = 0;
  while ( my $href = $schema_info->_first_program_details() ) {
    note $href->{ID};
    $count++;
  }
  is( $count, 53, q{53 programs} );
}

{
  my $schema_info;
  lives_ok {
    $schema_info = npg_pipeline::archive::file::SchemaInformation->new( {
      run_folder => $runfolder,
      runfolder_path => $analysis_runfolder_path,
      lane => 1,
      conf_path => q{data/config_files},
      domain => q{test},
      is_indexed => 1,
      pb_cal_log_file => q{t/data/logs/PB_cal_json_part.log},
      post_qseq_log_file => q{t/data/logs/post_qseq_json_part.log},
      timestamp => q{20101004-113033},
      recalibrated_path => $basecalls . q{/PB_cal},
      no_bsub => 1,
      aligner_cmd => q{t/bin/software/solexa/bin/bwa},
      samtools_cmd => q{t/bin/software/solexa/bin/samtools},
    } );
  } q{create object ok};

  throws_ok {
    $schema_info->_build_job_details();
  } qr{Unable[ ]to[ ]obtain[ ]bcl[ ]version}, q{_build_job_details runs ok};
  qx{cp t/data/logs/qseq_files_1234_all_20101006-1351216.out $basecalls/log/};

  is_deeply( $schema_info->_harold_alignment_files(), {
    DS => q{Performs an alignment of the lane's reference genome against the data, ready to produce a calibration table; Ref: no reference (need to use a control if found);},
    ID => q{recalibration_alignment},
    PN => q{pb_align},
    CL => q{not run},
  }, q{no log file for alignment, so no reference and needs control} );

  is_deeply( $schema_info->_harold_calibration_tables(), {
    DS => q{Creates the calibration table for the lane based on the alignment done; Lane calibration table made: False;},
    ID => q{recalibration_table_creation},
    PN => q{pb_calibration},
    CL => q{not run},
  }, q{no log file for calibration tables, so lane calibration table made is false} );

  is_deeply( $schema_info->_harold_recalibration(), {
   CL => q{not run},
   DS => q{Recalibrates the quality scores based on the calibration table used; Calibration table used: No Recalibration performed;},
   ID => q{recalibration},
   PN => q{pb_predictor}
  }, q{no log file for recalibration, so nothing recorded} );

  qx{cp t/data/logs/PB_cal_align_1234_1_paired_20101006-1351307.out $basecalls/PB_cal/log/};
  qx{cp t/data/logs/PB_cal_score_1234_1_1_20101006-143932.out $basecalls/PB_cal/log/};
  qx{cp t/data/logs/PB_cal_table_1234_1_1_201010068-152624.out $basecalls/PB_cal/log/};

  $schema_info->_clear_program_details();
  $schema_info->_lane_splitting_cache()->{1} = 'true';
  $schema_info->_lane_phix_splitting_cache()->{1} = 'true';
  lives_ok {
    $schema_info->_build_job_details();
  } q{_build_job_details runs ok};

  my $count = 0;
  while ( my $href = $schema_info->_first_program_details() ) {
    note $href->{ID};
    $count++;
  }
  is( $count, 47, q{47 programs} );

  lives_ok {
    $schema_info->generate_file();
  } q{file generated ok};

  my $info_retrieved;
  lives_ok {
    $info_retrieved = $schema_info->read_in_from_file();
  } q{file read ok};
  note explain $info_retrieved;
  is( scalar @{ $info_retrieved }, 47, q{47 program infos in stored file} );

  my $bsub_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J create_schema_of_run_1234_20101004-113033[1] -o /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/PB_cal/archive/log/create_schema_of_run_1234_20101004-113033.%I.%J.out 't/bin/software/solexa/bin/npg_pipeline_schema_information --aligner_cmd t/bin/software/solexa/bin/bwa --archive_path /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/PB_cal/archive --basecall_path /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls --bustard_path /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls --conf_path data/config_files --data_path /staging/IL2/analysis/123456_IL2_1234/Data --domain test --id_run 1234 --intensity_path /staging/IL2/analysis/123456_IL2_1234/Data/Intensities --internal_script_bin t/bin/software/solexa/bin --is_indexed  --name IL2_1234 --no_bsub  --pb_cal_log_file t/data/logs/PB_cal_json_part.log --pb_cal_path /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/PB_cal --post_qseq_log_file t/data/logs/post_qseq_json_part.log --recalibrated_path /staging/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/PB_cal --run_folder 123456_IL2_1234 --runfolder_path /staging/IL2/analysis/123456_IL2_1234 --samtools_cmd t/bin/software/solexa/bin/samtools --lane `echo $LSB_JOBINDEX`'};
  is( $util->drop_temp_part_from_paths( $schema_info->_generate_bsub_command( { required_job_completion => q{-w'done(123) && done(321)'} } ) ), $bsub_command, q{correct bsub command generated} );

  $schema_info->_clear_program_details();
  lives_ok {
    $schema_info->launch_bsub_job({});
  } q{launch bsub job ok};
}

#{
#  lives_ok {
#    qx{perl bin/npg_pipeline_schema_information --conf_path data/config_files --id_run 1234 --lane 2 --archive_path $analysis_runfolder_path/archive --pb_cal_log_file t/data/logs/PB_cal_json_part.log --post_qseq_log_file t/data/logs/post_qseq_json_part.log};
#  } q{script ran};
#  ok( ! $CHILD_ERROR, q{script ran ok} );
#}

1;
