# $Id: 20-archive_file_generation-gcfreq.t 12454 2011-01-26 10:59:47Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 9;
use lib qw{t};
use t::util;

use_ok('npg_pipeline::archive::file::generation::gcfreq');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();

$ENV{TEST_API_UTIL} = $util->apiutil();

{

  my $gcfreq_generator = npg_pipeline::archive::file::generation::gcfreq->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    conf_path => $conf_path,
    domain => q{test},
    verbose => 1,
  });
  isa_ok($gcfreq_generator, q{npg_pipeline::archive::file::generation::gcfreq}, q{$gcfreq_generator});

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};

  $util->create_analysis();

  my @jids;
  eval { @jids = $gcfreq_generator->generate($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak running generate});
  is(scalar@jids, 8, q{8 job ids returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $gcfreq_generator->_generate_bsub_command( $arg_refs ) );
  my $expected_command = q{bsub -q test -w'done(123) && done(321)' -J gcfreq_creation_1234_8_20090709-123456 -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/gcfreq_creation_8_20090709-123456.out 't/bin/software/solexa/bin/gcfreq_creator -q /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_8.fastq > /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_8.gcfreq'};
  is( $bsub_command, $expected_command, q{generated bsub command is correct});
}

{

  my $gcfreq_generator = npg_pipeline::archive::file::generation::gcfreq->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    lane => 4,
    conf_path => $conf_path,
    domain => q{test},
    verbose => 1,
  });
  isa_ok($gcfreq_generator, q{npg_pipeline::archive::file::generation::gcfreq}, q{$gcfreq_generator});

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};

  $util->create_analysis();

  my @jids;
  eval { @jids = $gcfreq_generator->generate($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak running generate});
  is(scalar@jids, 1, q{1 job id returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $gcfreq_generator->_generate_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -w'done(123) && done(321)' -J gcfreq_creation_1234_4_20090709-123456 -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/gcfreq_creation_4_20090709-123456.out 't/bin/software/solexa/bin/gcfreq_creator -q /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_4.fastq > /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_4.gcfreq'};
  is( $bsub_command, $expected_command, q{generated bsub command is correct} );
}

1;