# $Id: 20-archive_file_generation-srf.t 12454 2011-01-26 10:59:47Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 19;
use Test::Exception;
use lib qw{t};
use t::util;
use File::Slurp;

use_ok('npg_pipeline::archive::file::generation::srf');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
my $runfolder_path = $util->analysis_runfolder_path();
$ENV{TEST_USERAGENT} = $runfolder_path . q{/useragent};


{
  my $srf_generator;

  lives_ok { $srf_generator = npg_pipeline::archive::file::generation::srf->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    conf_path => $conf_path,
    domain => q{test},
#    verbose => 1,
  }); } q{no croak creating $srf_generator object};
  isa_ok($srf_generator, q{npg_pipeline::archive::file::generation::srf}, q{$srf_generator});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'}
  };

  $util->create_analysis({no_touch_files => 1});
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

  throws_ok {
    $srf_generator->generate($arg_refs);
  } qr{No[ ]qseq[ ]files[ ]found[ ]which[ ]give[ ]appropriate[ ]match}, q{no qseq files, so croaks out};

}

{
  my $srf_generator;

  lives_ok { $srf_generator = npg_pipeline::archive::file::generation::srf->new({
    run_folder => q{123456_IL2_1234},
    timestamp => q{20090709-123456},
    conf_path => $conf_path,
    domain => q{test},
#    verbose => 1,
    no_bsub => 1,
  }); } q{no croak creating $srf_generator object};

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};

  $util->create_analysis();
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

  my @jids;
  lives_ok { @jids = $srf_generator->generate($arg_refs); } q{no croak running generate};
  is(scalar@jids, 1, q{1 job id returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $srf_generator->_generate_srf_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -R 'select[mem>4000] rusage[mem=4000,nfs_12=3]' -M4000000 -w'done(123) && done(321)' -J srf_creation_1234_20090709-123456[1-8] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/srf_creation_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/illumina2srf  -b -filter-bad-reads -bustard-dir /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto -use_bases  Y37 -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.srf /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_1_qseq_custom.txt /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_2_qseq_custom.txt'};

  is( $bsub_command, $expected_command, q{generated srf bsub command is correct} );

  $bsub_command = $util->drop_temp_part_from_paths( $srf_generator->_generate_index_bsub_command($arg_refs) );

  $expected_command = q{bsub -q test -R 'select[mem>8000] rusage[mem=8000,nfs_12=2]' -M8000000 -w'done(123) && done(321)' -J srf_index_hash_1234_20090709-123456[1-8] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/srf_index_hash_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/srf_index_hash /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.srf'};
  is( $bsub_command, $expected_command, q{generated index bsub command is correct} );
}

{
  my $srf_generator;

  lives_ok { $srf_generator = npg_pipeline::archive::file::generation::srf->new({
    run_folder => q{123456_IL2_1234},
    use_bases => q{Y37,I6n,y37},
    timestamp => q{20090709-123456}, verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
    no_bsub => 1,
  }); } q{no croak creating $srf_generator object};

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};

  $util->create_tag_qseq();
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

  my @jids;
  lives_ok { @jids = $srf_generator->generate($arg_refs); } q{no croak running generate};
  is(scalar@jids, 1, q{1 job id returned});


  my $bsub_command = $util->drop_temp_part_from_paths( $srf_generator->_generate_srf_bsub_command($arg_refs) );
  my $expected_command_6n = q{bsub -q test -R 'select[mem>4000] rusage[mem=4000,nfs_12=3]' -M4000000 -w'done(123) && done(321)' -J srf_creation_1234_20090709-123456[1-8] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/srf_creation_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/illumina2srf  -b -filter-bad-reads -n %x:%y -bustard-dir /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto -use_bases  Y37,I6n,y37 -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.srf /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_1_1_qseq_custom.txt /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_2_1_qseq_custom.txt /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_3_1_qseq_custom.txt'};
  my $expected_command_7  = q{bsub -q test -R 'select[mem>4000] rusage[mem=4000,nfs_12=3]' -M4000000 -w'done(123) && done(321)' -J srf_creation_1234_20090709-123456[1-8] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/srf_creation_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/illumina2srf  -b -filter-bad-reads -n %x:%y -bustard-dir /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto -use_bases  Y37,I7,y37 -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.srf /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_1_1_qseq_custom.txt /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_2_1_qseq_custom.txt /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_3_1_qseq_custom.txt'};

  my $true = 0;
  if ( $bsub_command eq $expected_command_6n || $bsub_command eq $expected_command_7 ) {
    $true = 1;
  }
  ok( $true, q{generated srf bsub command is correct using either I6n or I7 in the use_bases string});

  $bsub_command = $util->drop_temp_part_from_paths( $srf_generator->_generate_index_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -R 'select[mem>8000] rusage[mem=8000,nfs_12=2]' -M8000000 -w'done(123) && done(321)' -J srf_index_hash_1234_20090709-123456[1-8] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/srf_index_hash_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/srf_index_hash /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.srf'};
  is($bsub_command, $expected_command, q{generated index bsub command is correct});
}

{
  my $srf_generator;

  lives_ok { $srf_generator = npg_pipeline::archive::file::generation::srf->new({
    run_folder => q{123456_IL2_1234},
    use_bases => q{Y37,I6n,y37},
    timestamp => q{20090709-123456},
    lane => 4,
    conf_path => $conf_path,
    domain => q{test},
    verbose => 1,
    no_bsub => 1,
  }); } q{no croak creating $srf_generator object};

  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  $util->create_tag_qseq();
  write_file( $ENV{TEST_USERAGENT}, $util->serialized_useragent() );

  my @jids;
  lives_ok { @jids = $srf_generator->generate($arg_refs); } q{no croak running generate with a lane provided};
  is(scalar@jids, 1, q{1 job id returned});

  `cp t/barcodes/five_bp.tags $runfolder_path/lane_4.tag`;

  $arg_refs->{type_of_lanes} = q{indexed_lanes};

  my $bsub_command = $util->drop_temp_part_from_paths( $srf_generator->_generate_srf_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -R 'select[mem>4000] rusage[mem=4000,nfs_12=3]' -M4000000 -w'done(123) && done(321)' -J srf_creation_1234_20090709-123456[4] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/srf_creation_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/illumina2srf  -b -filter-bad-reads -n %x:%y#%i -bustard-dir /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto -use_bases  Y37,I5n2,y37 -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.srf /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_1_1_qseq_custom.txt /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_2_1_qseq_custom.txt /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/s_`echo $LSB_JOBINDEX`_3_1_qseq_custom.txt'};
  is( $bsub_command, $expected_command, q{generated srf bsub command is correct} );

  $bsub_command = $util->drop_temp_part_from_paths( $srf_generator->_generate_index_bsub_command($arg_refs) );
  $expected_command = q{bsub -q test -R 'select[mem>8000] rusage[mem=8000,nfs_12=2]' -M8000000 -w'done(123) && done(321)' -J srf_index_hash_1234_20090709-123456[4] -o /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/srf_index_hash_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/srf_index_hash /staging/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`.srf'};
  is( $bsub_command, $expected_command, q{generated index bsub command is correct} );
}

1;
