# $Id: 20-archive_folder_WebCache.t 12454 2011-01-26 10:59:47Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';#tests => ;
use Test::Exception::LessClever;
use lib qw{t};
use t::util;
use Cwd;

BEGIN {
  use_ok(q{ npg_pipeline::archive::folder::WebCache} );
}

my $util = t::util->new();
my $info = $util->set_rta_staging_analysis_area();

{
  my $web_cache;
  $info->{id_run} = 1234;
  $info->{conf_path} = q{data/config_files};
  $info->{domain} = q{test};
  lives_ok {
    $web_cache = npg_pipeline::archive::folder::WebCache->new( $info );
  } q{WebCache object created ok};
  isa_ok( $web_cache, q{npg_pipeline::archive::folder::WebCache} , q{$web_cache} );

  my $webcache_archive_dir = $web_cache->archive_path() . q{/webcache};

  lives_ok {
    $web_cache->create_webcache_link();
  } q{create_webcache_link runs when nothing in the environment variable};

  ok( ! -d $webcache_archive_dir, q{no webcache directory in archive_path - no environment variable set} );

  $ENV{npg_common::request->cache_dir_var_name()} = q{/foo/bar/baz/spam/if/this/exists/i/will/be/shocked};

  lives_ok {
    $web_cache->create_webcache_link();
  } q{create_webcache_link runs when environment variable does not have a valid directory};

  ok( ! -d $webcache_archive_dir, q{no webcache directory in archive_path - environment variable not valid directory} );

  $ENV{npg_common::request->cache_dir_var_name()} = q{t/data};

  lives_ok {
    $web_cache->create_webcache_link();
  } q{create_webcache_link runs when environment does not include the runfolder_path};

  ok( ! -d $webcache_archive_dir, q{no webcache directory in archive_path - webcache path not in runfolder} );

  my $valid_webcache_path = $info->{runfolder_path} . q{/webcache_20101007-140205};
  qx{mkdir -p $valid_webcache_path/some_path};

  $ENV{npg_common::request->cache_dir_var_name()} = $valid_webcache_path;

  lives_ok {
    $web_cache->create_webcache_link();
  } q{create_webcache_link runs when environment does include the runfolder_path};

  ok( -d $webcache_archive_dir, q{webcache directory in archive_path - webcache path in runfolder} );

  my $cwd = getcwd();

  chdir $webcache_archive_dir;
  my $ls = qx{ls};
  chomp $ls;
  chdir $cwd;
  is( $ls, q{some_path}, q{correct link created} );

}
1;