# $Id: 25-analysis-CheckCifDif.t 12454 2011-01-26 10:59:47Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';#tests => ;
use Test::Exception;
use lib qw{t};
use t::util;

BEGIN {
  use_ok( q{npg_pipeline::analysis::CheckCifDif} );
}

my $util = t::util->new();
my $analysis_runfolder_path = $util->analysis_runfolder_path();

my $runfolder = $util->test_run_folder();

{
  my $checker;
  lives_ok {
    $checker = npg_pipeline::analysis::CheckCifDif->new( {
      run_folder => $runfolder,
      runfolder_path => $analysis_runfolder_path,
    } );
  } q{create check object ok};
  isa_ok( $checker, q{npg_pipeline::analysis::CheckCifDif}, q{$checker} );

  $util->multiplexed_rta_pre_gerald();

  isa_ok( $checker->intensities_config_xml(), q{XML::LibXML::Document} );

  is( $checker->first_cycle(), 1,   q{first cycle is correct} );
  is( $checker->last_cycle(),  160, q{last cycle is correct}  );

  throws_ok {
    $checker->_check_cif_files();
  } qr{6400[ ]error[ ]cif[ ]files[ ]found}, q{check_cif throws ok - no files and no fix_broken_files};

  throws_ok {
    $checker->_check_dif_files();
  } qr{6400[ ]error[ ]dif[ ]files[ ]found}, q{check_dif throws ok - no files and no fix_broken_files};
}

{
  my $checker = npg_pipeline::analysis::CheckCifDif->new( {
    run_folder => $runfolder,
    runfolder_path => $analysis_runfolder_path,
    fix_broken_files => 1,
  } );

  $util->multiplexed_rta_pre_gerald();
  qx{cp t/data/runfolder/Data/Intensities/single_end_3cycle_1tile_config.xml $analysis_runfolder_path/Data/Intensities/config.xml};

  is( $checker->first_cycle(), 1, q{first cycle is correct - 3 cycles} );
  is( $checker->last_cycle(),  3, q{last cycle is correct - 3 cycles}  );

  foreach my $lane ( 1..8 ) {
    foreach my $cycle ( 1..3 ) {
      qx{mkdir -p $analysis_runfolder_path/Data/Intensities/L00$lane/C$cycle.1};
    }
    qx{cp t/data/runfolder/Data/Intensities/demo_cif.cif $analysis_runfolder_path/Data/Intensities/L00$lane/C1.1/s_${lane}_1.cif};
  }
  lives_ok {
    $checker->_check_cif_files();
  } q{corrected all fine - fix_broken_files flag set to true};

}

1;