# $Id: 25-analysis-CheckStats.t 12454 2011-01-26 10:59:47Z ajb $
use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 8;
use Test::Exception;
use lib qw{t};
use t::util;
use DateTime;

BEGIN {
  use_ok( q{npg_pipeline::analysis::CheckStats} );
}
my $util = t::util->new();
my $analysis_runfolder_path = $util->analysis_runfolder_path();

my $runfolder = $util->test_run_folder();

$ENV{TEST_BIN} = q{t/bin/};

sub add_files {

  foreach my $lane ( 1..8 ) {
    foreach my $cycle ( 1..3 ) {
      qx{mkdir -p $analysis_runfolder_path/Data/Intensities/BaseCalls/L00$lane/C$cycle.1};
      qx{mkdir -p $analysis_runfolder_path/Data/Intensities/L00$lane/C$cycle.1};
      qx{touch $analysis_runfolder_path/Data/Intensities/L00$lane/C$cycle.1/s_${lane}_1.dif};
      qx{touch $analysis_runfolder_path/Data/Intensities/BaseCalls/L00$lane/C$cycle.1/s_${lane}_1.bcl};
    }
    qx{touch $analysis_runfolder_path/Data/Intensities/BaseCalls/s_${lane}_0001.filter};
  }
  return;
}

{
  my $checker;
  lives_ok {
    $checker = npg_pipeline::analysis::CheckStats->new({
      run_folder => $runfolder,
      runfolder_path => $analysis_runfolder_path,
      conf_path => q{data/config_files},
      domain => q{test},
    });
  } q{create checker object ok};
  isa_ok( $checker, q{npg_pipeline::analysis::CheckStats}, q{$checker} );

  $util->multiplexed_rta_pre_gerald();
  qx{cp t/data/runfolder/Data/Intensities/single_end_3cycle_1tile_config.xml $analysis_runfolder_path/Data/Intensities/config.xml};

  throws_ok {
    $checker->check_and_fix_stats();
  } qr{First[ ]file[ ]to[ ]be[ ]found[ ]missing[ ]is[ ]bcl[.][ ]Currently[ ]no[ ]way[ ]to[ ]fix[ ]this}, q{check_and_fix_stats throws ok - no expected files};

  add_files();

  throws_ok {
    $checker->check_and_fix_stats();
  } qr{missing[ ]expected[ ]files}, q{check_and_fix_stats throws ok - no expected stats files and croak_if_broken_cif_dif};

  $checker->_set_rta_version( q{1000.0.0} );

  throws_ok {
    $checker->check_and_fix_stats();
  } qr{First[ ]file[ ]to[ ]be[ ]found[ ]missing[ ]is[ ]filter_new_r1}, q{check_and_fix_stats throws ok - no expected stats files and croak_if_broken_cif_dif};

}

{
  my $checker = npg_pipeline::analysis::CheckStats->new({
    run_folder => $runfolder,
    runfolder_path => $analysis_runfolder_path,
    conf_path => q{data/config_files},
    domain => q{test},
    fix_broken_files => 1,
  });

  $util->multiplexed_rta_pre_gerald();
  qx{cp t/data/runfolder/Data/Intensities/single_end_3cycle_1tile_config.xml $analysis_runfolder_path/Data/Intensities/config.xml};
  add_files();

  lives_ok {
    $checker->check_and_fix_stats();
  } q{check_and_fix_stats runs ok - need stats files};
  lives_ok {
    $checker->check_and_fix_stats();
  } q{check_and_fix_stats runs ok - stats files found};
}

1;